#######################################################################
#
# NCM component for accounts daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 2.0.9 : 23/12/04 11:37
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::accounts;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;


local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/accounts";

    # Set this to 1 for debugging.  Normally we want to actually 
    # make the changes. 
    my $safemode = 0;
    
    # This code is taken from the edg-lcfg-auth component, changing
    # the access to the node configuration data and functionality to
    # add pool accounts.
    
    my %groups;  # hash containing old groups to delete, new groups to add and groups to modify
    my %users;   # hash containing old users to delete, new users to add and users to modify
    my ($groups_resource, $users_resource);
    my ($groupname, $username); 
    my ($line, @list, $primary, $retval, $val); # temp variables

    #########################
    # Start processing groups
    #########################

    # set all existing groups as "old"
    open GROUP,"/etc/group";
    while (($groupname) = split(/:/,<GROUP>)) {
        $groups{$groupname} = "old";
    }
    close GROUP;
    
    # process groups listed in auth resource "groups"
    # if already existing call groupmod and change %groups value to "mod"
    # otherwise call groupadd and set %groups value to "new"
    $groups_resource = $config->getElement("$base/groups");
    while ($groups_resource->hasNextElement()) {
        my $element = $groups_resource->getNextElement();

        # Collect options for this group.
        my @group_opt=();

        # The key is the name of the group.
        $groupname=$element->getName();

        my $prefix = "$base/groups/$groupname";

        # Add gid if specified.
        if ($config->elementExists("$prefix/gid")) {
            my $gid = $config->getElement("$prefix/gid")->getValue();
            push(@group_opt,'-g'.$gid);
        }

        # Define command/value necessary. 
        my $cmd = '/usr/sbin/groupadd';
        my $state = 'new';
        if (defined($groups{$groupname}) && $groups{$groupname} ne "new") {
            $cmd = '/usr/sbin/groupmod';
            $state = 'mod';
        }

        # Run the command to modify or create a group.
        $groups{$groupname} = $state;
        $self->log(join(" ",$cmd,@group_opt)." ".$groupname);
        unless ($safemode) {
            $retval=system($cmd,@group_opt,$groupname);
            $self->warn("Failed to modify/add group ".$groupname) if ($retval);
        }
    }
    
    # removal of old groups is done after processing users because we don't
    # remove groups that are still primary group for a user
    
    
    #########################
    # Start processing users
    #########################

    # set root password
    if ($config->elementExists("$base/rootpwd")) {
        my $rootpwd=$config->getValue("$base/rootpwd");
        if (defined $rootpwd && $rootpwd ne '') {
            if ($safemode) {
                $self->log("Not changing root password while in safe mode");
            } else {
                $retval=system("/usr/sbin/usermod","-p",$rootpwd,"root");
                $self->warn("Failed to set root password") if ($retval);
            }
        }
    }
    
    # set all existing users as "old"
    # only users on which useradd/mod/del can operate (so method: files)
    open PASSWD,"/etc/passwd";
    while (($username) = split(/:/,<PASSWD>) ) {
        $users{$username} = "old";
    }
    close PASSWD;
    
    # process users listed in auth resource "users"
    # if already existing call usermod and change %users value to "mod"
    # otherwise call useradd and set %users value to "new"
    $users_resource = $config->getElement("$base/users");
    while ($users_resource->hasNextElement()) {
        my $element = $users_resource->getNextElement();

        # Collect options for this user.
        my @user_opt=();

        # The key is the name of the user.
        $username=$element->getName();

        my $prefix = "$base/users/$username";

        # The home directory for the user. 
        if ($config->elementExists("$prefix/homeDir")) {
            my $opt = $config->getValue("$prefix/homeDir");
            push(@user_opt,'-d',$opt);
        }
        
        # The primary group for the user. 
        if ($config->elementExists("$prefix/groups/0")) {
            my $opt = $config->getValue("$prefix/groups/0");
            push(@user_opt,'-g',$opt);
        }

        # All of the supplemental groups. 
        my $i = 1;
        my @suppgroups = ();
        while ($config->elementExists("$prefix/groups/$i")) {
            my $opt = $config->getValue("$prefix/groups/$i");
            push(@suppgroups,$opt);
	    $i++;
        }
        if ($#suppgroups >= 0) {
            push(@user_opt,'-G',join(',',@suppgroups));
        } 
        
        # The shell for the user. 
        if ($config->elementExists("$prefix/shell")) {
            my $opt = $config->getValue("$prefix/shell");
            push(@user_opt,'-s',$opt);
        }

        # The base uid for the account.  Treat this specially
        # so that group accounts are handled correctly. 
        my $baseuid = undef;
        if ($config->elementExists("$prefix/uid")) {
            $baseuid = $config->getValue("$prefix/uid");
        }

        # Comment for the account. 
        if ($config->elementExists("$prefix/comment")) {
            my $opt = $config->getValue("$prefix/comment");
            push(@user_opt,'-c',$opt);
        }
        
        # The password for the account. 
        if ($config->elementExists("$prefix/password")) {
            my $opt = $config->getValue("$prefix/password");
            push(@user_opt,'-p',$opt);
        }

        # get options used only for creating new users (userpwd and usernotcreatehome)
        my @useradd_opt=();
        
        # Whether to create directory or not. 
	my $createHome = 1;
        if ($config->elementExists("$prefix/createHome")) {
	    $createHome = $config->getValue("$prefix/createHome");
            push(@useradd_opt,'-M') unless $createHome;
        }

        # Whether to create ssh keys or not.
	my $createKeys = 0;
        if ($config->elementExists("$prefix/createKeys")) {
	    $createKeys = $config->getValue("$prefix/createKeys");
        }

        # Get the number of pool accounts to create. 
        my $poolSize = 0;
        if ($config->elementExists("$prefix/poolSize")) {
            $poolSize = $config->getValue("$prefix/poolSize");
        }

        # Now add or modify the user. 
        my $limit = ($poolSize>0) ? ($poolSize-1) : 0;
        for my $i (0 .. $limit) {
            my $uname = $username . pool_suffix($i,($poolSize-1));

            # Add the name and the uid as the last set of options. 
            my @opts = ($uname);
            if (defined($baseuid)) {
                my $uid = $baseuid + $i;
                unshift @opts, $uid;
                unshift @opts, '-u';
            }

            # Define necessary command and value. 
            my $cmd = '/usr/sbin/useradd';
            my $state = 'new';
            if (defined($users{$uname}) && $users{$uname} ne 'new') {
                $cmd = '/usr/sbin/usermod';
                $state = 'mod';
            } else {
                push(@user_opt,@useradd_opt);
            }

            # Run the command to modify/add user. 
            $users{$uname} = $state;
            $self->log(join(' ',$cmd,@user_opt,@opts));
            unless ($safemode) {
                $retval=system($cmd,@user_opt,@opts);
                $self->warn("Failed modify/add user ".$uname) if ($retval);
            }

	    # Determine if ssh keys should be generated for this user.
	    if ($createKeys && $createHome && 
		($state eq 'new') && !$safemode) {
		$self->keygen($uname);
	    }
        }
    }
    
    # delete users
    # special case: ignore users starting with '+' or '-' to allow use of NIS
    while (($username, $val) = each(%users)) {
        my $firstchar=substr($username, 0, 1);
        if ($val eq 'old' && $firstchar ne '+' && $firstchar ne '-') {
            $self->log("/usr/sbin/userdel ".$username);
            unless ($safemode) {
                $retval=system("/usr/sbin/userdel ".$username);
                $self->warn("Failed to delete user ".$username) if ($retval);
            }
        }
    }
    
    # delete groups 
    # special case: ignore groups starting with '+' or '-' to allow use of NIS 
    while (($groupname, $val) = each(%groups)) {
        my $firstchar=substr($groupname, 0, 1);
        if ($val eq 'old' && $firstchar ne '+' && $firstchar ne '-') {

            my $primary = 0;

            my $gid = getgrnam($groupname);
            if (defined($gid) && $gid ne '') {
                open PASSWD,"/etc/passwd";
                while (my ($name, $pwd, $uid, $testgid) = split(/:/,<PASSWD>) ) {
                    $primary = 1 if ($gid == $testgid);
                }
                close PASSWD;
            }

            $self->log("/usr/sbin/groupdel ".$groupname) if ($primary);
            if (!$safemode && $primary) {
                $retval=system("/usr/sbin/groupdel ".$groupname);
                $self->warn("Failed to delete group ".$groupname) if ($retval);
            } else {
                $self->warn("group $groupname not deleted because it's a primary group for some user");
            }
        }
    }
    
    # shadow passwords
    my $shadow = 0;
    if ($config->elementExists("$base/shadowpwd")) {
        $shadow = $config->getValue("$base/shadowpwd");
    }

    if ($safemode) {
        $self->log("Not activating shadow passwords while in safe mode") if $shadow;
    } elsif ($shadow) {
        $self->log("/usr/sbin/pwconv");
        $self->log("/usr/sbin/grpconv");
        $retval=system("/usr/sbin/pwconv");
        $retval+=system("/usr/sbin/grpconv");
        $self->Warn("Failed to set shadow passwords") if ($retval);
    } else {
        $self->log("/usr/sbin/pwunconv");
        $self->log("/usr/sbin/grpunconv");
        $retval=system("/usr/sbin/pwunconv");
        $retval+=system("/usr/sbin/grpunconv");
        $self->Warn("Failed to unset shadow passwords") if ($retval);
    }
    
    # set file permmissions
    chmod(0644, "/etc/passwd");
    chmod(0644, "/etc/group");
 
    return 1;
}

# Creates a pool account suffix.  Takes two arguments
# the index to use and the maximum value.  If the 
# maximum value is less than zero, an empty string
# is returned. 
sub pool_suffix {

    my ($index, $max) = @_;

    my $v = '';

    if ($max >= 0) {
        $v = "$index";
        while (length("$v")<length($max)) {
            $v = "0$v";
        }
    }
    return $v;
}

# Generate ssh keys for a user.  This was lifted from the poolaccounts
# LCFG object written by Steve Traylen.  
sub keygen {

    my ($self,$user) = @_;

    # Get the UID, GID, and home directory of user.  There seems to be
    # a race condition when getpwnam is called the first time causes
    # the information not to be returned.  So it is called twice below
    # to avoid this. 
    my @dummy = getpwnam($user);
    my ($uid, $gid, $home) = (getpwnam($user))[2,3,7];

    unless (defined($uid) && defined($gid) && defined($home)) {
	$self->warn("User doesn't exist: $user");
	return;
    }

    ####################################################
    # Set the EUID, and EGID to cope with root_squashes.
    $) = $gid;
    $> = $uid;
    ###################################################
    # Create the .ssh directory.
    mkdir("$home/.ssh") unless (-d "$home/.ssh") ;
    ###################################################
    # Create the ssh 2 keypair.
    if (! -f "$home/.ssh/id_rsa") {
	$> = 0 ; $) = 0 ;
	$self->Info("Generating RSA2 key for $user") ;
	$) = $gid; $> = $uid;
	my @cmd = ('/usr/bin/ssh-keygen','-t','rsa','-q','-N','',
                   '-C',$user,'-f',"$home/.ssh/id_rsa") ;
	my $retval = system(@cmd) ;
	if ( $retval != 0 ) {
	    $> = 0 ; $) = 0 ;
	    $self->Fail("Failed to generate key for $user.") ;
	}
    }
    ###################################################
    # Create the ssh 1 keypair.
    if (! -f "$home/.ssh/identity") {
	$> = 0 ; $) = 0 ;
	$self->Info("Generating RSA1 key for $user") ;
	$) = $gid ; $> = $uid ;
	my @cmd = ('/usr/bin/ssh-keygen','-t','rsa1','-q','-N','',
                   '-C',$user,'-f',"$home/.ssh/identity") ;
	my $retval = system(@cmd) ;
	if ( $retval != 0 ) {
	    $> = 0 ; $) = 0 ;
	    $self->Fail("Failed to generate key for $user.") ;
	}
    }
    ##################################################
    # Create the authorised keys file.
    open(AUTH,">$home/.ssh/authorized_keys") ;
    open(PUB,"<$home/.ssh/identity.pub") ;
    while(<PUB>) {
	print AUTH $_ ;
    }
    close(PUB);
    open(PUB,"<$home/.ssh/id_rsa.pub");
    while(<PUB>) {
	print AUTH $_ ;
    }
    close(PUB);
    close(AUTH) ;
    ##################################################
    # Set the EUID and EGID back to what they should be.
    $> = 0 ;
    $) = 0 ;
}


1;      # Required for PERL modules
